 /**********************************************************************
 *
 *
 *		            Rover Toolkit
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: base64.h
 *
 * Description: MIME-compliant base 64 encoding library header.
 *
 *
 * Global Functions: None
 * Global Variables: None
 *
 * 
 *
 ***********************************************************************
 *   Copyright 1995-1998
 *   Alan deLespinasse and Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#ifndef base64_h
#define base64_h

#include <stdio.h>
#include <stdlib.h>

/* The caller of these functions should never have to worry about
   what's inside these structures. In other words, don't allocate or
   deallocate the buf field of b64fromstate; the initializer and end
   functions take care of it. */

typedef struct _b64tostate {
  int phase;   /* 1,2,3, or 4 (number of bytes made) mod 4 */
  int column;  /* number of chars written on current row */
  char b,p;    /* input bytes being dealt with */
} b64tostate;

typedef struct _b64fromstate {
  int done;    /* MAY be 1 if end of input reached */
  int shift;   /* amount to shift bytes by */
  char prevc;  /* last char read */
  short nextc; /* current char */
  char *buf;   /* space for encoded data */
  size_t bufsize,    /* size of buf */
  bufofs;	     /* current offset in buf */
} b64fromstate;

/* To encode: call b64toinit to fill in a b64tostate structure. Then
   call b64towrite as many times as necessary to write all data to a file.
   Finish by calling b64towriteend (does not close file). */

extern void b64toinit(b64tostate *s);
extern size_t b64towrite(char *ptr, size_t n, FILE *f, b64tostate *s);
extern void b64towriteend(FILE *f, b64tostate *s);

/* To decode: initialize with b64fromreadinit on an open FILE*. Then
   b64fromread and b64fromfgets (rougly like the read and fgets functions)
   can be intermixed to read blocks or lines. Call b64fromend when done. */

extern void b64fromreadinit(FILE *f, b64fromstate *s);
extern size_t b64fromread(char *buf, size_t n, FILE *f, b64fromstate *s);
extern size_t b64fromfgets(char *buf, size_t n, FILE *f, b64fromstate* s);
extern void b64fromend(b64fromstate *s);

#endif /* base64_h */
