 /**********************************************************************
 *
 *
 *		            Rover Toolkit
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: tclIndex.c
 *
 * Description: Tcl autoloader module
 *
 * Global Functions: None
 * Global Variables: None
 *
 ***********************************************************************
 *   Copyright 1995-1998
 *   Anthony D. Joseph and Joshua A. Tauber
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.
 ***********************************************************************/

#include <errno.h>
#include <time.h>
#include <stdlib.h>
#include <sys/param.h>
#include "rover.h"
#include "rovertcp.h"
#include "tclIndex.h"

void tcl_importdata(char *object);

static char line[INITSIZE];

void tcl_importdata(char *object)
{   
  int   length = strlen("Rover__tclIndex__");

  if (RoverDebug)
    fprintf(stderr, "tclIndex.c:tcl_importdata:\"%s\":%lu\n", object,
	    state->UserTime);

  if (strncmp(object, "Rover__tclIndex__", length)) {
    sprintf(line,
	    "Tcl autoloader server: Non-tclIndex object: %s [tcl_importdata:tclIndex.c]\n",
	    object);
    errorExit(line);
  }
  
  if (createTclInterp(state) < 0)  {
    state->status = 500;
    errorExit("Tcl autoloader server: Unable to create tcl interpreter");
  }
  Tcl_LoadLibrary("autoload.tcl.obj", state);

  if (Tcl_VarEval(globalTclInterp, ROVER_LOAD_INDEX, &object[length], NULL)) {
    sprintf(line,
	    "Tcl autoloader server: Unable to load index: %s [tcl_importdata:tclIndex.c]\n",
	    globalTclInterp->result);
    errorExit(line);
  }

  state->dvtime = atol(globalTclInterp->result);
  if (state->UserTime == 0) {
    if (RoverDebug)
      fprintf(stderr, "New object: %ld [tcl_importdata:tclIndex.c]\n", 
	    state->dvtime);
    state->ObjType = ROVER_TCL_TK;
    state->Cacheable = ROVER_BEST_EFFORT;
  } else if (state->dvtime == state->UserTime) {
    if (RoverDebug)
    fprintf(stderr, "Verified object: %ld [tcl_importdata:tclIndex.c]\n", 
	    state->dvtime);
    state->ObjType = ROVER_TCL_TK;
    state->Cacheable = ROVER_VERIFIED;
  } else {
    if (RoverDebug)
      fprintf(stderr, "Updated object: %ld [tcl_importdata:tclIndex.c]\n", 
	      state->dvtime);
    state->ObjType = ROVER_TCL_TK_UPDATE;
    state->Cacheable = ROVER_BEST_EFFORT;
  }
}
